@@ -66,11 +66,12 @@ typedef struct {
 	uchar bcmode  : 1;
 	uchar minimal : 1;
 	uchar repl    : 1;
-	uchar rsvd    : 3; /* Reserved for future usage */
+	uchar calc    : 1;
+	uchar rsvd    : 2; /* Reserved for future usage */
 	uchar loglvl  : 2;
 } settings;
 
-static char *VERSION = "2.1";
+static char *VERSION = "2.2";
 static char *units[] = {"b", "kib", "mib", "gib", "tib", "kb", "mb", "gb", "tb"};
 static char *logarr[] = {"ERROR", "WARNING", "INFO", "DEBUG"};
 
@@ -83,7 +84,7 @@ static char uint_buf[UINT_BUF_LEN];
 static char float_buf[FLOAT_BUF_LEN];
 
 static Data lastres = {"\0", 0};
-static settings cfg = {0, 0, 0, 0, INFO};
+static settings cfg = {0, 0, 0, 0, 0, INFO};
 
 static void debug_log(const char *func, int level, const char *format, ...)
 {
@@ -171,8 +172,10 @@ static size_t bstrlcpy(char *dest, const char *src, size_t n)
 static int try_bc(char *expr)
 {
 	pid_t pid;
-	int pipe_pc[2], pipe_cp[2], ret;
-	size_t len, count = 0;
+	int pipe_pc[2], pipe_cp[2];
+	size_t len;
+	ssize_t ret;
+	char *ptr = cfg.calc ? "calc" : "bc";
 
 	if (!expr) {
 		if (curexpr)
@@ -205,24 +208,26 @@ static int try_bc(char *expr)
 		dup2(pipe_cp[1], STDOUT_FILENO); // Give stdout to parent
 		dup2(pipe_cp[1], STDERR_FILENO); // Give stderr to parent
 
-		ret = execlp("bc", "bc", "-q", (char*) NULL);
+		ret = execlp(ptr, ptr, (char*) NULL);
 		log(ERROR, "execlp() failed!\n");
 		exit(ret);
 	}
 
-	/* parent */
-	if (write(pipe_pc[1], "scale=10\n", 9) != 9) {
-		log(ERROR, "write(1)! [%s]\n", strerror(errno));
-		exit(-1);
+	if (!cfg.calc) {
+		/* parent */
+		if (write(pipe_pc[1], "scale=10\n", 9) != 9) {
+			log(ERROR, "write(1)! [%s]\n", strerror(errno));
+			exit(-1);
+		}
 	}
 
-	if (write(pipe_pc[1], "last=", 5) != 5) {
+	if (write(pipe_pc[1], "r=", 2) != 2) {
 		log(ERROR, "write(2)! [%s]\n", strerror(errno));
 		exit(-1);
 	}
 
 	if (lastres.p[0]) {
-		ret = strlen(lastres.p);
+		ret = (ssize_t)strlen(lastres.p);
 		if (write(pipe_pc[1], lastres.p, ret) != ret) {
 			log(ERROR, "write(3)! [%s]\n", strerror(errno));
 			exit(-1);
@@ -239,7 +244,7 @@ static int try_bc(char *expr)
 		exit(-1);
 	}
 
-	ret = strlen(expr);
+	ret = (ssize_t)strlen(expr);
 	if (write(pipe_pc[1], expr, ret) != ret) {
 		log(ERROR, "write(6)! [%s]\n", strerror(errno));
 		exit(-1);
@@ -261,14 +266,21 @@ static int try_bc(char *expr)
 	buffer[ret] = '\0';
 
 	if (buffer[0] != '(') {
-		printf("%s", buffer);
-		len = bstrlcpy(lastres.p, buffer, NUM_LEN);
+		ptr = buffer;
+		while (isspace(*ptr)) /* calc results have space before them */
+			++ptr;
+
+		printf("%s", ptr);
+		len = bstrlcpy(lastres.p, ptr, NUM_LEN);
 
 		/* remove newline appended at the end of result by bc */
 		(len >= 2) ? (len -= 2) : (len = 0);
 		lastres.p[len] = '\0';
 
+#ifdef TRIM_DECIMAL
 		/* Trim the decimal part, if any */
+		size_t count = 0;
+
 		while (count < len) {
 			if (lastres.p[count] == '.') {
 				lastres.p[count] = '\0';
@@ -277,6 +289,7 @@ static int try_bc(char *expr)
 
 			++count;
 		}
+#endif
 		lastres.unit = 0;
 		log(DEBUG, "result: %s %d\n", lastres.p, lastres.unit);
 		return 0;
@@ -1162,7 +1175,7 @@ Webpage: https://github.com/jarun/bcal\n", VERSION);
 
 static int bstricmp(const char *s1, const char *s2)
 {
-	while (*s1 && (tolower(*s1) == tolower(*s2))) {
+	while ((int)*s1 && (tolower((int)*s1) == tolower((int)*s2))) {
 		++s1;
 		++s2;
 	}
@@ -1430,7 +1443,7 @@ static maxuint_t eval(queue **front, queue **rear, int *out)
 		dequeue(front, rear, &arg);
 
 		/* Check if arg is an operator */
-		if (strlen(arg.p) == 1 && !isdigit(arg.p[0])) {
+		if (strlen(arg.p) == 1 && !isdigit((int)arg.p[0])) {
 			pop(&est, &raw_b);
 			pop(&est, &raw_a);
 
@@ -1609,7 +1622,7 @@ static int issign(char c)
 }
 
 /* Check if a char is operator or not */
-static int isoperator(char c)
+static int isoperator(int c)
 {
 	switch (c) {
 	case '+':
@@ -1649,16 +1662,16 @@ static void strstrip(char *s)
 	if (!s || !*s)
 		return;
 
-	int len = strlen(s) - 1;
+	int len = (int)strlen(s) - 1;
 
 	if (s[len] == '\n')
 		--len;
-	while (len >= 0 && (isspace(s[len]) || s[len] == '\"' || s[len] == '\''))
+	while (len >= 0 && (isspace((int)s[len]) || s[len] == '\"' || s[len] == '\''))
 		--len;
 	s[len + 1] = '\0';
 
 	len = 0;
-	while (s[len] && (isspace(s[len]) || s[len] == '\"' || s[len] == '\''))
+	while (s[len] && (isspace((int)s[len]) || s[len] == '\"' || s[len] == '\''))
 		++len;
 
 	if (len) {
@@ -1678,7 +1691,7 @@ static void removeinnerspaces(char *s)
 
 	while (*s != '\0') {
 		/* We should not combine 0xn b*/
-		if (!isspace(*s) || (*(s + 1) == 'b')) {
+		if (!isspace((int)*s) || (*(s + 1) == 'b')) {
 			*p = *s;
 			++p;
 		}
@@ -1726,17 +1739,17 @@ static char *fixexpr(char *exp, int *unitless)
 			return NULL;
 		}
 
-		if (isoperator(exp[i]) && isalpha(exp[i + 1]) && (exp[i + 1] != 'r')) {
+		if (isoperator((int)exp[i]) && isalpha((int)exp[i + 1]) && (exp[i + 1] != 'r')) {
 			log(ERROR, "invalid expression\n");
 			free(parsed);
 			return NULL;
 		}
 
-		if ((isdigit(exp[i]) && isoperator(exp[i + 1])) ||
-		    (isoperator(exp[i]) && (isdigit(exp[i + 1]) ||
-		     isoperator(exp[i + 1]))) ||
-		    (isalpha(exp[i]) && isoperator(exp[i + 1])) ||
-		    (isoperator(exp[i]) && (exp[i + 1] == 'r'))) {
+		if ((isdigit((int)exp[i]) && isoperator((int)exp[i + 1])) ||
+		    (isoperator((int)exp[i]) && (isdigit((int)exp[i + 1]) ||
+		     isoperator((int)exp[i + 1]))) ||
+		    (isalpha((int)exp[i]) && isoperator((int)exp[i + 1])) ||
+		    (isoperator((int)exp[i]) && ((int)exp[i + 1] == 'r'))) {
 			if (exp[i] == '<' || exp[i] == '>') { /* handle shift operators << and >> */
 				if (prev != exp[i] && exp[i] != exp[i + 1]) {
 					log(ERROR, "invalid operator %c\n", exp[i]);
@@ -1805,10 +1818,10 @@ static int convertunit(char *value, char *unit, ulong sectorsz)
 	}
 
 	if (!unit) {
-		int unitchars = 0, len = strlen(value);
+		int unitchars = 0, len = (int)strlen(value);
 
 		while (len) {
-			if (!isalpha(value[len - 1]))
+			if (!isalpha((int)value[len - 1]))
 				break;
 
 			++unitchars;
@@ -1923,7 +1936,8 @@ static int evaluate(char *exp, ulong sectorsz)
 	char *expr = fixexpr(exp, &ret);  /* Make parsing compatible */
 	char *ptr;
 
-	log(DEBUG, "expr: %s\n", expr);
+	if (expr)
+		log(DEBUG, "expr: %s\n", expr);
 
 	if (expr == NULL) {
 		if (ret)
@@ -2014,6 +2028,9 @@ int main(int argc, char **argv)
 	int opt = 0, operation = 0;
 	ulong sectorsz = SECTOR_SIZE;
 
+	if (getenv("BCAL_USE_CALC"))
+		cfg.calc = TRUE;
+
 	opterr = 0;
 	rl_bind_key('\t', rl_insert);
 
@@ -2029,7 +2046,7 @@ int main(int argc, char **argv)
 		case 'f':
 			operation = 1;
 
-			if (tolower(*optarg) == 'c') {
+			if (tolower((int)*optarg) == 'c') {
 				maxuint_t lba = 0;
 
 				if (chs2lba(optarg + 1, &lba)) {
@@ -2038,7 +2055,7 @@ int main(int argc, char **argv)
 					printhex_u128(lba);
 					printf("\n\n");
 				}
-			} else if (tolower(*optarg) == 'l') {
+			} else if (tolower((int)*optarg) == 'l') {
 				t_chs chs;
 
 				if (lba2chs(optarg + 1, &chs)) {
@@ -2062,7 +2079,10 @@ int main(int argc, char **argv)
 			break;
 		case 'b':
 			cfg.bcmode = 1;
-			strncpy(prompt, "bc> ", 5);
+			if (cfg.calc)
+				strncpy(prompt, "calc> ", 7);
+			else
+				strncpy(prompt, "bc> ", 5);
 			break;
 		case 'd':
 			cfg.loglvl = DEBUG;
@@ -2137,8 +2157,12 @@ int main(int argc, char **argv)
 				case 'b':
 					cfg.bcmode ^= 1;
 					if (cfg.bcmode) {
-						printf("bc vars: scale = 10, ibase = 10, last = r\n");
-						strncpy(prompt, "bc> ", 5);
+						if (cfg.calc)
+							strncpy(prompt, "calc> ", 7);
+						else {
+							printf("bc vars: scale = 10, ibase = 10\n");
+							strncpy(prompt, "bc> ", 5);
+						}
 					} else
 						strncpy(prompt, "bcal> ", 7);
 
